/* wptUTF8.cpp - UTF8 conversation
 *	Copyright (C) 2002, 2004, 2005, 2006, 2009 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "wptTypes.h"
#include "wptErrors.h"


/**
 * Converts the given intput string, which is encoded with the locale 
 * setting, into UTF-8 representation.
 */
char*
native_to_utf8 (const char *string)
{
    wchar_t *result;
    char *native;
    int n;

    n = MultiByteToWideChar (GetACP (), 0, string, -1, NULL, 0);
    if (n < 0)
	return NULL;

    result = new wchar_t[n+1];
    if (!result)
	BUG (0);

    n = MultiByteToWideChar (GetACP (), 0, string, -1, result, n);
    if (n < 0) {
	free_if_alloc (result);
	return NULL;
    }

    n = WideCharToMultiByte (CP_UTF8, 0, result, -1, NULL, 0, NULL, NULL);
    if (n < 0)
	return NULL;

    native = new char[n+1];
    if (!native)
	BUG (0);

    n = WideCharToMultiByte (CP_UTF8, 0, result, -1, native, n, NULL, NULL);
    if (n < 0) {
	free_if_alloc (result);
	return NULL;
    }

    free_if_alloc (result);
    //native[n] = '\0';        
    return native;
}


/**
 * Converts the given string, which is encoded in UTF-8,
 * into the locale setting.
 */
char*
utf8_to_native (const char *string)
{
    wchar_t *result;
    char *native;    
    int n;

    n = MultiByteToWideChar (CP_UTF8, 0, string, -1, NULL, 0);
    if (n < 0)
	return NULL;

    result = new wchar_t[n+1];
    if (!result)
	BUG (0);

    n = MultiByteToWideChar (CP_UTF8, 0, string, -1, result, n);
    if (n < 0) {
	free_if_alloc (result);
	return NULL;
    }

    n = WideCharToMultiByte (GetACP (), 0, result, -1, NULL, 0, NULL, NULL);
    if (n < 0)
	return NULL;

    native = new char[n+1];
    if (!native)
	BUG (0);

    n = WideCharToMultiByte (GetACP (), 0, result, -1, native, n, NULL, NULL);
    if (n < 0) {
	free_if_alloc (result);
	return NULL;
    }
    
    free_if_alloc (result);
    //native[n] = '\0';
    return native;
}


/**
 * Returns -1 if the given string contains any 8-bit characters.
 * This is a helper to decide when to use UTF8 encoding.
 */
int
is_8bit_string (const char *str)
{
    for (size_t i = 0; i < strlen (str); i++) {
	if (str[i] & 0x80)
	    return -1;
    }
    return 0;
}
